@@ -13,22 +13,39 @@ Other functions let you create a PM drawing as string or into a PM buffer.
 Execute the script to see some test drawings."""
 
 from reportlab.graphics.shapes import *
-from reportlab.graphics.renderbase import StateTracker, getStateDelta, renderScaledDrawing
+from reportlab.graphics.renderbase import getStateDelta, renderScaledDrawing
 from reportlab.pdfbase.pdfmetrics import getFont, unicode2T1
-from math import sin, cos, pi, ceil
-from reportlab.lib.utils import getStringIO, getBytesIO, open_and_read, isUnicode
-from reportlab import rl_config, ascii
+from reportlab.lib.utils import isUnicode
+from reportlab import rl_config
 from .utils import setFont as _setFont, RenderPMError
 
 import os, sys
+from io import BytesIO, StringIO
+from math import sin, cos, pi, ceil
 
-try:
-    from reportlab.graphics import _renderPM
-except ImportError as errMsg:
-    raise ImportError("""No module named _renderPM\nit may be badly or not installed!
+def _getPMBackend(backend=None):
+    if not backend: backend = rl_config.renderPMBackend
+    if backend=='_renderPM':
+        try:
+            from reportlab.graphics import _renderPM as M
+        except ImportError as errMsg:
+            raise ImportError("""No module named _renderPM
+it may be badly or not installed!
 You may need to install development tools
 or seek advice at the users list see
 https://pairlist2.pair.net/mailman/listinfo/reportlab-users""")
+    elif 'cairo' in backend.lower():
+        try:
+            import rlPyCairo as M
+        except ImportError as errMsg:
+            raise ImportError("""cannot import desired renderPM backend rlPyCairo
+Seek advice at the users list see
+https://pairlist2.pair.net/mailman/listinfo/reportlab-users""")
+    else:
+        raise RenderPMError(f'Invalid renderPM backend, {backend}')
+    return M
+
+_pmBackend = _getPMBackend(rl_config.renderPMBackend)
 
 def _getImage():
     try:
@@ -91,7 +108,7 @@ class _PMRenderer(Renderer):
 
     def initState(self,x,y):
         deltas = self._tracker._combined[-1]
-        deltas['transform'] = self._canvas._baseCTM[0:4]+(x,y)
+        deltas['transform'] = deltas['ctm'] = self._canvas._baseCTM[0:4]+(x,y)
         self._tracker.push(deltas)
         self.applyState()
 
@@ -256,8 +273,7 @@ def _convert2pil1(im):
 def _saveAsPICT(im,fn,fmt,transparent=None):
     im = _convert2pilp(im)
     cols, rows = im.size
-    #s = _renderPM.pil2pict(cols,rows,im.tostring(),im.im.getpalette(),transparent is not None and Color2Hex(transparent) or -1)
-    s = _renderPM.pil2pict(cols,rows,(im.tobytes if hasattr(im,'tobytes') else im.tostring)(),im.im.getpalette())
+    s = _pmBackend.pil2pict(cols,rows,(im.tobytes if hasattr(im,'tobytes') else im.tostring)(),im.im.getpalette())
     if not hasattr(fn,'write'):
         with open(os.path.splitext(fn)[0]+'.'+fmt.lower(),'wb') as f:
             f.write(s)
@@ -284,18 +300,12 @@ class PMCanvas:
         self.ctm = self._baseCTM
 
     @staticmethod
-    def _getGState(w, h, bg, backend='_renderPM', fmt='RGB24'):
+    def _getGState(w, h, bg, backend=None, fmt='RGB24'):
+        mod = _getPMBackend(backend)
         if backend=='_renderPM':
-            return _renderPM.gstate(w,h,bg=bg)
-        elif backend=='rlPyCairo':
-            try:
-                from rlPyCairo import GState
-            except ImportError:
-                raise RenderPMError('cannot import rlPyCairo; perhaps it needs to be installed!')
-            else:
-                return GState(w,h,bg,fmt=fmt)
-        else:
-            raise RenderPMError('Invalid backend, %r, given to PMCanvas' % backend)
+            return mod.gstate(w,h,bg=bg)
+        elif 'cairo' in backend.lower():
+            return mod.GState(w,h,bg,fmt=fmt)
 
     def _drawTimeResize(self,w,h,bg=None):
         if bg is None: bg = self._bg
@@ -306,7 +316,7 @@ class PMCanvas:
         for k in A.keys():
             A[k] = getattr(gs,k)
         del gs, self._gs
-        gs = self.__dict__['_gs'] = _renderPM.gstate(w,h,bg=bg)
+        gs = self.__dict__['_gs'] = _pmBackend.gstate(w,h,bg=bg)
         for k in A.keys():
             setattr(self,k,A[k])
         gs.setFont(fN,fS)
@@ -342,20 +352,9 @@ class PMCanvas:
         elif fmt in ('PCT','PICT'):
             return _saveAsPICT(im,fn,fmt,transparent=configPIL.get('transparent',None))
         elif fmt in ('PNG','BMP', 'PPM'):
-            if fmt=='PNG':
-                try:
-                    from PIL import PngImagePlugin
-                except ImportError:
-                    import PngImagePlugin
-            elif fmt=='BMP':
-                try:
-                    from PIL import BmpImagePlugin
-                except ImportError:
-                    import BmpImagePlugin
+            pass
         elif fmt in ('JPG','JPEG'):
             fmt = 'JPEG'
-        elif fmt in ('GIF',):
-            pass
         else:
             raise RenderPMError("Unknown image kind %s" % fmt)
         if fmt=='TIFF':
@@ -384,7 +383,7 @@ class PMCanvas:
             markfilename(fn,ext=fmt)
 
     def saveToString(self,fmt='GIF'):
-        s = getBytesIO()
+        s = BytesIO()
         self.saveToFile(s,fmt=fmt)
         return s.getvalue()
 
@@ -693,7 +692,7 @@ def drawToFile(d,fn,fmt='GIF', dpi=72, bg=0xffffff, configPIL=None, showBoundary
     c.saveToFile(fn,fmt)
 
 def drawToString(d,fmt='GIF', dpi=72, bg=0xffffff, configPIL=None, showBoundary=rl_config._unset_,backend=rl_config.renderPMBackend):
-    s = getBytesIO()
+    s = BytesIO()
     drawToFile(d,s,fmt=fmt, dpi=dpi, bg=bg, configPIL=configPIL,backend=backend)
     return s.getvalue()
 
@@ -737,12 +736,12 @@ def test(outDir='pmout', shout=False):
         msg = 'Problem drawing %s fmt=%s file'%(name,fmt)
         if shout or verbose>2: print(msg)
         errs.append('<br/><h2 style="color:red">%s</h2>' % msg)
-        buf = getStringIO()
+        buf = StringIO()
         traceback.print_exc(file=buf)
         errs.append('<pre>%s</pre>' % escape(buf.getvalue()))
 
     #print in a loop, with their doc strings
-    for (drawing, docstring, name) in getAllTestDrawings(doTTF=hasattr(_renderPM,'ft_get_face')):
+    for (drawing, docstring, name) in getAllTestDrawings(doTTF=hasattr(_pmBackend,'ft_get_face')):
         i = names[name] = names.setdefault(name,0)+1
         if i>1: name += '.%02d' % (i-1)
         if argv and name not in argv: continue
